#include "LoggingService.h"
#include "ErisLogReciever.h"

// Include system headers here
#include <string>
#include <list>
#include <stdio.h>
#include <sstream>
//#include <varargs.h> //TODO: Needed by unix?

#include <sigc++/object_slot.h>

namespace Ember {

LoggingService* LoggingService::theInstance = NULL;

Service::Status LoggingService::start ()
{
    setRunning (true);
    setStatus (Service::OK);
    return Service::OK;
}


void LoggingService::log (const char *message, ...)
{
    va_list vl;
    va_start (vl, message);
    logVarParam ("", -1, INFO, message, vl);
    va_end (vl);
}


void LoggingService::log (const char *file, const char *message, ...)
{
    va_list vl;
    va_start (vl, message);
    logVarParam (file, -1, INFO, message, vl);
    va_end (vl);
}



void LoggingService::log (const char *file, const int line,
                                const char *message, ...)
{
    va_list vl;
    va_start (vl, message);
    logVarParam (file, line, INFO, message, vl);
    va_end (vl);
}


void LoggingService::log (const MessageImportance importance,
                                const char *message, ...)
{
    va_list vl;
    va_start (vl, message);
    logVarParam ("", -1, importance, message, vl);
    va_end (vl);
}

void LoggingService::log (const char *file,
                                const MessageImportance importance,
                                const char *message, ...)
{
    va_list vl;
    va_start (vl, message);
    logVarParam (file, -1, importance, message, vl);
    va_end (vl);
}

void LoggingService::log (const char *file, const int line,
                                const MessageImportance importance,
                                const char *message, ...)
{
    va_list vl;
    va_start (vl, message);
    logVarParam (file, line, importance, message, vl);
    va_end (vl);
}

void LoggingService::logVarParam (const char *file, const int line,
                                        const MessageImportance importance,
                                        const char *message, va_list argptr)
{
    char Buffer[MESSAGE_BUFFER_SIZE];
    vsprintf ((char *) Buffer, message, argptr);
    sendMessage (std::string ((char *) Buffer), file, line, importance);
}


LoggingService & LoggingService::slog (const std::string & file,
                                                   const int line,
                                                   const MessageImportance
                                                   importance)
{
    myFile = file;
    myLine = line;
    myImportance = importance;
    return *this;
}

LoggingService & LoggingService::
slog (const MessageImportance importance)
{
    myImportance = importance;
    return *this;
}

LoggingService & LoggingService::slog (const std::string & file,
                                                   const MessageImportance
                                                   importance)
{
    myFile = file;
    myImportance = importance;
    return *this;
}

LoggingService & LoggingService::slog (const std::string & file,
                                                   const int line)
{
    myFile = file;
    myLine = line;
    return *this;
}

LoggingService & LoggingService::slog (const std::string & file)
{
    myFile = file;
    return *this;
}




void LoggingService::addObserver (Observer * observer)
{
    //test on already existing observer
    for (ObserverList::iterator i = myObserverList.begin ();
         i != myObserverList.end (); i++)
    {
        if (*i == observer)
        {
            return;
        }
    }

    //no existing observer, add a new                    
    myObserverList.push_back (observer);
}

int LoggingService::removeObserver (Observer * observer)
{
    for (ObserverList::iterator i = myObserverList.begin ();
         i != myObserverList.end (); i++)
    {
        if (*i == observer)
        {
            myObserverList.erase (i);
            return 0;
        }
    }

    return -1;
}

LoggingService::HexNumber LoggingService::
hexNumber (const int intDecimal)
{
    HexNumber intHex;
    intHex.myNumber = intDecimal;
    return intHex;
}

LoggingService & LoggingService::operator<< (const std::
                                                         string & stringToAdd)
{
    myMessage += stringToAdd;
    return *this;
}

LoggingService & LoggingService::operator<< (const unsigned int uintToAdd)
{
	std::stringstream ss;
	ss << uintToAdd;
    myMessage += ss.str();
    
    return *this;
}

LoggingService & LoggingService::operator<< (const int intToAdd)
{
	std::stringstream ss;
	ss << intToAdd;
    myMessage += ss.str();
    
    return *this;
}

LoggingService & LoggingService::
operator<< (const HexNumber & intHexToAdd)
{
    char buffer[NUMBER_BUFFER_SIZE];
    sprintf (buffer, "%x", intHexToAdd.myNumber);
    myMessage += buffer;
    return *this;
}

LoggingService & LoggingService::
operator<< (const double doubleToAdd)
{
	std::stringstream ss;
	ss << doubleToAdd;
    myMessage += ss.str();
    return *this;
}

LoggingService & LoggingService::
operator<< (const unsigned long ulongToAdd)
{
	std::stringstream ss;
	ss << ulongToAdd;
    myMessage += ss.str();
    return *this;
}


void LoggingService::operator<< (const EndMessageEnum endMessage)
{
    sendMessage (myMessage, myFile, myLine, myImportance);

    myMessage = "";
    myFile = "";
    myLine = -1;
    myImportance = INFO;
}

LoggingService::LoggingService () : Service()
{
    //set service properties

    setName ("Logging");
    setDescription ("Eases message writing and distribution.");


    //set all option values to not specified
    myMessage = "";
    myFile = "";
    myLine = -1;
    myImportance = INFO;

	mErisLogReciever = std::auto_ptr<ErisLogReciever>(new ErisLogReciever(*this));
}


LoggingService::~LoggingService ()
{

}


void LoggingService::sendMessage (const std::string & message,
                                        const std::string & file,
                                        const int line,
                                        const MessageImportance importance)
{
    time_t currentTime;
    time (&currentTime);

    for (ObserverList::iterator i = myObserverList.begin ();
         i != myObserverList.end (); i++)
    {
        if ((int) importance >= (int) (*i)->getFilter ())
        {
            (*i)->onNewMessage (message, file, line, importance, currentTime);
        }
    }
}

}
